@@ -253,6 +253,9 @@ static PyObject *eval(PyObject *self, PyObject *args) {
   js::ESClass cls = js::ESClass::Other;   // placeholder if `rval` is not a JSObject
   if (rval->isObject()) {
     JS::GetBuiltinClass(GLOBAL_CX, JS::RootedObject(GLOBAL_CX, &rval->toObject()), &cls);
+    if (JS_ObjectIsBoundFunction(&rval->toObject())) {
+      cls = js::ESClass::Function; // In SpiderMonkey 115 ESR, bound function is no longer a JSFunction but a js::BoundFunctionObject.
+    }
   }
   bool rvalIsFunction = cls == js::ESClass::Function;   // function object
   bool rvalIsString = rval->isString() || cls == js::ESClass::String;   // string primitive or boxed String object
@@ -268,6 +271,17 @@ static PyObject *eval(PyObject *self, PyObject *args) {
   }
 }
 
+static PyObject *waitForEventLoop(PyObject *Py_UNUSED(self), PyObject *Py_UNUSED(_)) {
+  PyObject *waiter = PyEventLoop::_locker->_queueIsEmpty; // instance of asyncio.Event
+
+  // Making sure it's attached to the current event-loop
+  PyEventLoop loop = PyEventLoop::getRunningLoop();
+  if (!loop.initialized()) return NULL;
+  PyObject_SetAttrString(waiter, "_loop", loop._loop);
+
+  return PyObject_CallMethod(waiter, "wait", NULL);
+}
+
 static PyObject *isCompilableUnit(PyObject *self, PyObject *args) {
   StrType *buffer = new StrType(PyTuple_GetItem(args, 0));
   const char *bufferUtf8;
@@ -289,6 +303,7 @@ static PyObject *isCompilableUnit(PyObject *self, PyObject *args) {
 
 PyMethodDef PythonMonkeyMethods[] = {
   {"eval", eval, METH_VARARGS, "Javascript evaluator in Python"},
+  {"wait", waitForEventLoop, METH_NOARGS, "The event-loop shield. Blocks until all asynchronous jobs finish."},
   {"isCompilableUnit", isCompilableUnit, METH_VARARGS, "Hint if a string might be compilable Javascript"},
   {"collect", collect, METH_VARARGS, "Calls the spidermonkey garbage collector"},
   {NULL, NULL, 0, NULL}
@@ -305,90 +320,6 @@ struct PyModuleDef pythonmonkey =
 
 PyObject *SpiderMonkeyError = NULL;
 
-// Implement the `setTimeout` global function
-//    https://developer.mozilla.org/en-US/docs/Web/API/setTimeout
-//    https://html.spec.whatwg.org/multipage/timers-and-user-prompts.html#dom-settimeout
-static bool setTimeout(JSContext *cx, unsigned argc, JS::Value *vp) {
-  JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
-
-  // Ensure the first parameter is a function
-  // We don't support passing a `code` string to `setTimeout` (yet)
-  JS::HandleValue jobArgVal = args.get(0);
-  bool jobArgIsFunction = jobArgVal.isObject() && js::IsFunctionObject(&jobArgVal.toObject());
-  if (!jobArgIsFunction) {
-    JS_ReportErrorNumberASCII(cx, nullptr, nullptr, JSErrNum::JSMSG_NOT_FUNCTION, "The first parameter to setTimeout()");
-    return false;
-  }
-
-  // Get the function to be executed
-  // FIXME (Tom Tang): memory leak, not free-ed
-  JS::RootedObject *thisv = new JS::RootedObject(cx, JS::GetNonCCWObjectGlobal(&args.callee()));   // HTML spec requires `thisArg` to be the global object
-  JS::RootedValue *jobArg = new JS::RootedValue(cx, jobArgVal);
-  // `setTimeout` allows passing additional arguments to the callback, as spec-ed
-  if (args.length() > 2) {   // having additional arguments
-    // Wrap the job function into a bound function with the given additional arguments
-    //    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind
-    JS::RootedVector<JS::Value> bindArgs(cx);
-    (void)bindArgs.append(JS::ObjectValue(**thisv));   /** @todo XXXwg handle return value */
-    for (size_t j = 2; j < args.length(); j++) {
-      (void)bindArgs.append(args[j]);   /** @todo XXXwg handle return value */
-    }
-    JS::RootedObject jobArgObj = JS::RootedObject(cx, &jobArgVal.toObject());
-    JS_CallFunctionName(cx, jobArgObj, "bind", JS::HandleValueArray(bindArgs), jobArg);   // jobArg = jobArg.bind(thisv, ...bindArgs)
-  }
-  // Convert to a Python function
-  PyObject *job = pyTypeFactory(cx, thisv, jobArg)->getPyObject();
-
-  // Get the delay time
-  //  JS `setTimeout` takes milliseconds, but Python takes seconds
-  double delayMs = 0;   // use value of 0 if the delay parameter is omitted
-  if (args.hasDefined(1)) { JS::ToNumber(cx, args[1], &delayMs); }   // implicitly do type coercion to a `number`
-  if (delayMs < 0) { delayMs = 0; }   // as spec-ed
-  double delaySeconds = delayMs / 1000;   // convert ms to s
-
-  // Schedule job to the running Python event-loop
-  PyEventLoop loop = PyEventLoop::getRunningLoop();
-  if (!loop.initialized()) return false;
-  PyEventLoop::AsyncHandle handle = loop.enqueueWithDelay(job, delaySeconds);
-
-  // Return the `timeoutID` to use in `clearTimeout`
-  args.rval().setDouble((double)PyEventLoop::AsyncHandle::getUniqueId(std::move(handle)));
-
-  return true;
-}
-
-// Implement the `clearTimeout` global function
-//    https://developer.mozilla.org/en-US/docs/Web/API/clearTimeout
-//    https://html.spec.whatwg.org/multipage/timers-and-user-prompts.html#dom-cleartimeout
-static bool clearTimeout(JSContext *cx, unsigned argc, JS::Value *vp) {
-  using AsyncHandle = PyEventLoop::AsyncHandle;
-  JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
-  JS::HandleValue timeoutIdArg = args.get(0);
-
-  args.rval().setUndefined();
-
-  // silently does nothing when an invalid timeoutID is passed in
-  if (!timeoutIdArg.isInt32()) {
-    return true;
-  }
-
-  // Retrieve the AsyncHandle by `timeoutID`
-  int32_t timeoutID = timeoutIdArg.toInt32();
-  AsyncHandle *handle = AsyncHandle::fromId((uint32_t)timeoutID);
-  if (!handle) return true; // does nothing on invalid timeoutID
-
-  // Cancel this job on Python event-loop
-  handle->cancel();
-
-  return true;
-}
-
-static JSFunctionSpec jsGlobalFunctions[] = {
-  JS_FN("setTimeout", setTimeout, /* nargs */ 2, 0),
-  JS_FN("clearTimeout", clearTimeout, 1, 0),
-  JS_FS_END
-};
-
 PyMODINIT_FUNC PyInit_pythonmonkey(void)
 {
   if (!PyDateTimeAPI) { PyDateTime_IMPORT; }
@@ -440,11 +371,6 @@ PyMODINIT_FUNC PyInit_pythonmonkey(void)
 
   autoRealm = new JSAutoRealm(GLOBAL_CX, *global);
 
-  if (!JS_DefineFunctions(GLOBAL_CX, *global, jsGlobalFunctions)) {
-    PyErr_SetString(SpiderMonkeyError, "Spidermonkey could not define global functions.");
-    return NULL;
-  }
-
   JS_SetGCCallback(GLOBAL_CX, handleSharedPythonMonkeyMemory, NULL);
   JS_DefineProperty(GLOBAL_CX, *global, "debuggerGlobal", debuggerGlobal, JSPROP_READONLY);
 
@@ -493,6 +419,9 @@ PyMODINIT_FUNC PyInit_pythonmonkey(void)
     return NULL;
   }
 
+  // Initialize event-loop shield
+  PyEventLoop::_locker = new PyEventLoop::Lock();
+
   PyObject *internalBindingPy = getInternalBindingPyFn(GLOBAL_CX);
   if (PyModule_AddObject(pyModule, "internalBinding", internalBindingPy) < 0) {
     Py_DECREF(internalBindingPy);
